#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# create_paper_wallet                                                          #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program creates a QR code for a specified public key and private key.   #
# It then creates an image of a Bitcoin paper wallet. This program loads the   #
# keys from a Python file (keys.py by default) which defines the string        #
# variables key_public and key_private.                                        #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help         display help message
    --version          display version and exit

    --keys=FILEPATH    keys file          [default: keys.py]
    --wallet=FILEPATH  paper wallet file  [default: paper_wallet.png]
"""

import docopt
import os
import PIL.Image
import PIL.ImageDraw
import PIL.ImageFont
import re
import sys

import pyqrcode

name    = "create_paper_wallet"
version = "2017-12-25T1844Z"
logo    = None

def main(options):

    filepath_keys         = options["--keys"]
    filepath_paper_wallet = options["--wallet"]

    # QR codes

    filepath_keys = os.path.expanduser(filepath_keys)

    if os.path.isfile(filepath_keys):

        with open(filepath_keys, "r") as file_keys:
            file_string = file_keys.read()
            keys = {}
            exec(file_string, keys)
            key_public  = keys["key_public"]
            key_private = keys["key_private"]

        if key_public:
            qr = pyqrcode.create(key_public)
            qr.png("key_public.png", scale = 50, quiet_zone = 0)
            resize_image(filename = "key_public.png", size = 602)

        if key_private:
            qr = pyqrcode.create(key_private)
            qr.png("key_private.png", scale = 50, quiet_zone = 0)
            resize_image(filename = "key_private.png", size = 602)

    else:

        print("no keys file {filepath_keys} found".format(filepath_keys = filepath_keys))
        sys.exit()

    # paper wallet image

    if all([
        os.path.isfile("key_public.png"),
        os.path.isfile("key_private.png"),
        os.path.isfile("Bitcoin_paper_wallet_template.png"),
        os.path.isfile("consolas.ttf"),
        os.path.isfile("FuturaLT-Light.ttf")
        ]):

        key_public_text  = "public key:\n\n"  + key_public
        key_private_text = "private key:\n\n" + key_private

        key_public_text  = hard_wrap_text(text = key_public_text,  width = 26)
        key_private_text = hard_wrap_text(text = key_private_text, width = 26)

        image_paper_wallet = PIL.Image.open("Bitcoin_paper_wallet_template.png")

        draw       = PIL.ImageDraw.Draw(image_paper_wallet)
        font_keys  = PIL.ImageFont.truetype("consolas.ttf", 42)
        font_title = PIL.ImageFont.truetype("FuturaLT-Light.ttf", 110)

        draw.text((415, 89),   "BITCOIN ADDRESS KEYS", (0, 0, 0), font = font_title)
        draw.text((51, 969),   key_public_text,        (0, 0, 0), font = font_keys)
        draw.text((1349, 969), key_private_text,       (0, 0, 0), font = font_keys)

        image_key_public  = PIL.Image.open("key_public.png")
        image_key_private = PIL.Image.open("key_private.png")

        image_paper_wallet.paste(image_key_public,  (51, 319))
        image_paper_wallet.paste(image_key_private, (1349, 319))

        image_paper_wallet.save("paper_wallet.png")

    else:

        print("missing files for creating paper wallet".format(filepath_keys = filepath_keys))
        sys.exit()

def resize_image(
    filename = None,
    size     = 602
    ):

    image    = PIL.Image.open(filename)
    fraction = size / float(image.size[0])
    hsize    = int(float(image.size[1]) * float(fraction))
    image    = image.resize((size, hsize), PIL.Image.ANTIALIAS)
    image.save(filename) 

def hard_wrap_text(
    text  = None,
    width = None
    ):

    return re.sub(r'(.{' + str(width) + '})', '\\1\n', text)

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# loop_display_arbitrage_data                                                  #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program loop displays Kraken LocalBitcoins UK arbitrage data.           #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help          display help message
    --version           display version and exit

    --CSV=FILEPATH      filepath of CSV data          [default: arbitrage_Kraken_LocalBitcoins_UK.csv]

    --interval=INT      time between recordings (s)   [default: 60]

    --volume_EUR=FLOAT  EUR volume used in arbitrage  [default: 9000]
"""

from __future__ import division
import docopt
import os
import sys
import textwrap
import time

import denarius
import pandas as pd

name    = "loop_display_arbitrage_data"
version = "2018-01-23T1746Z"
logo    = None

def main(options):

    filepath_CSV = os.path.expanduser(options["--CSV"])
    interval     = int(options["--interval"])
    volume_EUR   = float(options["--volume_EUR"])

    while True:
        print(denarius.printout_arbitrage_Kraken_LocalBitcoins_UK(
            filepath_CSV = filepath_CSV,
            volume_EUR   = volume_EUR
        ))
        time.sleep(interval)
        os.system("clear")

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# Kraken                                                                       #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program provides Kraken utilities.                                      #
#                                                                              #
# copyright (C) 2018 William Breaden Madden, Liam Moore                        #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################
"""

import time
import os
import sys

import krakenex

name    = "Kraken"
version = "2018-02-06T1826Z"

def Kraken_access_ok():
    """
    Check Kraken API access.
    """
    if last_price_XBT_EUR():
        return True
    else:
        return False

def start_API(
    filepath_credentials = "~/.kraken"
    ):
    filepath_credentials = os.path.expanduser(filepath_credentials)
    global kraken
    try:
        time.sleep(2)
        kraken = krakenex.API()
    except:
        print("Kraken API error")
    kraken.load_key(filepath_credentials)

def last_price_XBT_EUR(
    convert_value_strings_to_floats = True
    ):
    """
    Return the last price of Bitcoin in Euros.
    """
    time.sleep(2)
    try:
        request_prices_XBT_EUR = kraken.query_public("Ticker", {"pair": "XXBTZEUR"})
        if request_prices_XBT_EUR["error"] == []:
            last_price = request_prices_XBT_EUR["result"]["XXBTZEUR"]["c"][0]
            if convert_value_strings_to_floats:
                last_price = float(last_price)
            return last_price
        else:
            try:
                print(request_balances["error"][0])
            except:
                pass
            return None
    except:
        print("Kraken API error")

def balances_of_currencies(
    convert_value_strings_to_floats = True
    ):
    """
    Return the current balances of currencies held.
    """
    try:
        time.sleep(2)
        request_balances = kraken.query_private("Balance")
        if request_balances["error"] == []:
            balances = request_balances["result"]
            if convert_value_strings_to_floats:
                balances = {key: float(value) for key, value in list(balances.items())}
            return balances
        else:
            try:
                print(request_balances["error"][0])
            except:
                pass
            return None
    except:
        print("Kraken API error")
        return None

def buy_XBT_for_EUR(
    volume_in_BTC = None
    ):
    """
    Place a buy order for Bitcoin at the current market EUR price.
    """
    if isinstance(volume_in_BTC, int):
        volume_in_BTC = float(volume_in_BTC)
    if volume_in_BTC >= 0.002:
        print("place buy order for {volume} BTC at current market price".format(
            volume = volume_in_BTC
        ))
        try:
            time.sleep(2)
            request_buy = kraken.query_private(
                "AddOrder",
                {
                    "pair"            : "XXBTZEUR",
                    "type"            : "buy",
                    "ordertype"       : "market",
                    "volume"          : volume_in_BTC
                }
            )
            if request_buy["error"] == []:
                return request_buy["result"]
            else:
                try:
                    print(request_buy["error"][0])
                except:
                    pass
                return None
        except:
            print("Kraken API error")
            return None
    elif volume_in_BTC < 0.002:
        print("minimum order is 0.002 BTC")
        return None
    else:
        print("no volume specified")
        return None

def send_XBT(
    amount      = None,
    address_key = None
    ):
    """
    Send Bitcoin to an address, the key for which has been verified on Kraken.
    """
    if amount and address_key:
        print("send {amount} BTC to address key {key}".format(
            amount = amount,
            key    = address_key
        ))
        try:
            time.sleep(2)
            request_withdraw = kraken.query_private(
                "Withdraw",
                {
                    "asset" : "XBT",
                    "key"   : address_key,
                    "amount": amount
                }
            )
            if request_withdraw["error"] == []:
                return request_withdraw["result"]
            else:
                try:
                    print(request_withdraw["error"][0])
                except:
                    pass
                return None
        except:
            print("Kraken API error")
            return None
    else:
        print("no amount or no address key specified")
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# denarius_loop_save_KanoPool                                                  #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program loop records KanoPool address data.                             #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help           display help message
    --version            display version and exit

    --addresses=TEXT     address or comma-separated addresses
    --interval=FLOAT     loop time interval (s)        [default: 60]
    --file_CSV=FILENAME  filename for saving CSV data  [default: KanoPool.csv]
"""

from bs4 import BeautifulSoup
import docopt
import json
from collections import OrderedDict
import os
import requests
import sys
import time

name    = "denarius_loop_save_KanoPool"
version = "2017-07-10T1020Z"
logo    = name

def main(options):

    addresses    =       options["--addresses"]
    interval     = float(options["--interval"])
    filename_CSV =       options["--file_CSV"]

    if not addresses:
        print("error -- no address specified")
        sys.exit()

    addresses = addresses.split(",")

    variables = OrderedDict([
        ("hashrate5m",  "hash_rate_TH/s_5m"),
        ("hashrate1hr", "hash_rate_TH/s_1hr"),
        ("hashrate1d",  "hash_rate_TH/s_1d"),
        ("lastupdate",  "last_update"),
        ("workers",     "workers"),
        ("shares",      "shares"),
        ("bestshare",   "best_share")
    ])

    # if new file, add header
    if not os.path.isfile(filename_CSV):
        with open(filename_CSV, "a") as file_CSV:
            line = ["address"]
            line.extend(list(variables.values()))
            file_CSV.write(",".join(line))

    while True:

        for address in addresses:

            try:
                URL         = "https://www.kano.is/address.php?a=" + address
                data_string = requests.get(URL).text
                soup        = BeautifulSoup(data_string, "lxml")
                data_JSON   = json.loads(soup.findAll("body")[0].get_text())

                line = [address]
                print("\naddress: {address}".format(address = address))
                for variable_key in list(variables.keys()):
                    if "hashrate" in variable_key:
                        value = data_JSON[variable_key].rstrip("T")
                    else:
                        value = data_JSON[variable_key]
                    print("{variable}: {value}".format(
                        variable = variables[variable_key],
                        value    = value
                    ))
                    line.append(str(value))
                with open(filename_CSV, "ab") as file_CSV:
                    file_CSV.write("\n" + ",".join(line))
            except:
                pass

        print("\nwait {interval} s".format(interval = interval))
        time.sleep(interval)

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python

"""
################################################################################
#                                                                              #
# buy_BTC_for_EUR_last_price_on_Kraken                                         #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program buys Bitcoin for the last market price in Euros on Kraken.      #
#                                                                              #
# copyright (C) 2018 William Breaden Madden                                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    --volume=FLOAT  volume in BTC [default: 0.002]
"""

import docopt
import sys

import denarius.Kraken

name    = "buy_BTC_for_EUR_last_price_on_Kraken"
version = "2018-01-31T0010Z"

def main(options):

    volume_in_BTC = float(options["--volume"])
    denarius.Kraken.start_API()
    print(denarius.Kraken.buy_XBT_for_EUR(volume_in_BTC = volume_in_BTC))

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    main(options)



# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# RBS                                                                          #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program provides banking and other utilities.                           #
#                                                                              #
# copyright (C) 2017 William Breaden Madden                                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################
"""

import decimal
import docopt
import os
import re
import sys
import time

import numpy as np
import pandas as pd
from selenium import webdriver
from selenium.webdriver.common.keys import Keys

name    = "RBS"
version = "2018-01-18T1452Z"

def account_status(
    filepath_credentials               = "~/.rbs",
    convert_currency_strings_to_floats = True,
    log_out                            = True,
    close_driver                       = True,
    sleep_time                         = 6
    ):

    """
    Access RBS online banking using credentials specified or from a file. Get
    the balance and transactions for the account specified. Return a dictionary
    of the balance and a DataFrame of recent transactions. Return None if there
    is an error.

    The form of the DataFrame of recent transactions is something like the
    following:

         date                                   description  amount
    0  13 Jan                             CALL REF.NO. 1234  -500.0
    1  05 Jan    L. RON HUBBARD\nDigital/Mobile Transaction  -950.0
    2  04 Jan      J. H. CHRIST\nDigital/Mobile Transaction    -5.0
    3  02 Jan                   M. F. GOD\nAutomated Pay In   100.0
    4  02 Jan                     GANESHA\nAutomated Pay In   250.0
    5  02 Jan                  G. V. FUNK\nAutomated Pay In   950.0
    6  02 Jan                    DR. ZEUS\nAutomated Pay In   100.0
    7  28 Dec                       DAGDA\nAutomated Pay In    90.0
    8  27 Dec                     WEI DAI\nAutomated Pay In    20.0
    9  27 Dec            C. COLLINS-TOOTH\nAutomated Pay In   500.0

    The date is not specified in the web interface with time, so this field is
    not converted to a datetime.
    """

    # access credentials from file
    filepath_credentials = os.path.expanduser(filepath_credentials)
    if not os.path.isfile(filepath_credentials):
        print("no credentials file {filepath} found".format(filepath = filepath_credentials))
        sys.exit()
    with open(filepath_credentials, "r") as file_credentials:
        credentials_string = file_credentials.read()
    credentials = {}
    exec(credentials_string, credentials)
    customer_number = credentials["customer_number"]
    PIN             = credentials["PIN"]
    passcode        = credentials["passcode"]
    account_code    = credentials["account_code"]
    try:
        driver = webdriver.Firefox()
        driver.get("http://personal.rbs.co.uk")
        # login button
        button_login = driver.find_element_by_class_name("gnav-login-button")
        button_login.click()
        time.sleep(sleep_time)
        # enter customer number
        driver.switch_to_frame("ctl00_secframe")
        element = driver.find_element_by_name("ctl00$mainContent$LI5TABA$CustomerNumber_edit")
        element.click()
        element.send_keys(customer_number)
        element.send_keys(Keys.RETURN)
        time.sleep(sleep_time)
        # enter PIN
        for box in ["A", "B", "C"]:
            element = driver.find_element_by_name("ctl00$mainContent$Tab1$LI6PPE" + box + "_edit")
            element.click()
            element.send_keys(PIN[find_ordinal(driver.find_element_by_id("ctl00_mainContent_Tab1_LI6DDAL" + box + "Label").text) - 1])
        # enter passcode
        for box in ["D", "E", "F"]:
            element = driver.find_element_by_name("ctl00$mainContent$Tab1$LI6PPE" + box + "_edit")
            element.click()
            element.send_keys(passcode[find_ordinal(driver.find_element_by_id("ctl00_mainContent_Tab1_LI6DDAL" + box + "Label").text) - 1])
        # next button
        element = driver.find_element_by_name("ctl00$mainContent$Tab1$next_text_button_button")
        element.click()
        time.sleep(sleep_time)
        # get statement element and balance
        statement = driver.find_element_by_id("Account_" + account_code)
        balance   = statement.text.split()[-2] # string with pounds symbol
        # get transactions table
        statement.click()
        time.sleep(1)
        transactions = driver.find_element_by_class_name("tranTable")
        # convert transactions table to DataFrame
        df = pd.DataFrame(columns = ["date"])
        for row in transactions.find_elements_by_tag_name("tr"):
            columns = row.find_elements_by_tag_name("td")
            columns = [column.text.rstrip("-") for column in columns]
            if columns:
                df = df.append(
                    {
                        "date":        columns[0],
                        "description": columns[1],
                        "amount":      columns[2]
                    },
                    ignore_index = True
                )
        df = df[["date", "description", "amount"]]
        if log_out:
            driver.get("https://www.rbsdigital.com/ServiceManagement/RedirectOutOfService.aspx?targettag=destination_ExitService&amp;secstatus=0")
        if close_driver:
            driver.quit()
        if convert_currency_strings_to_floats:
            balance      = currency_string_to_float(balance)
            df["amount"] = df.apply(lambda x: currency_string_to_float(x["amount"]), axis = 1)
        return {
            "balance":      balance,
            "transactions": df
        }
    except:
        try:
            driver.quit()
        except:
            pass
        return None

def loop_save_transactions_to_CSV(
    filepath_CSV = "RBS.csv",
    sleep_time   = 6,
    interval     = 30
    ):

    """
    Get current transactions online. Open previously-recorded transactions, if
    any, and merge them with the current transactions to avoid duplications.
    Save the merged transactions to CSV. Do this in a loop.
    """

    while True:
        try:
            # get transactions
            print("access account")
            status = account_status(sleep_time = sleep_time)
            df = status["transactions"]
            df["balance"] = np.nan
            # add balance only to top transaction
            df.at[0, "balance"] = status["balance"]
            print("--------------------------------------------------------------------------------")
            print("current account status:\n")
            print(df)
            # check existing recorded data to avoid saving previously-saved data
            if os.path.isfile(filepath_CSV):
                #print("merge current account status with previously-recorded states")
                df_old = pd.read_csv(filepath_CSV)
                #print("--------------------------------------------------------------------------------")
                #print("previous account status:\n")
                #print(df_old)
                df = pd.concat([df_old, df], ignore_index = True).drop_duplicates().reset_index(drop = True)
                #print("--------------------------------------------------------------------------------")
                #print("merged account status:\n")
                #print(df)
                header = False
            else:
                header = True
            with open(filepath_CSV, "a") as file_CSV:
                print("save transactions to CSV {filepath}".format(filepath = filepath_CSV))
                df.to_csv(file_CSV, header = header, index = False)
        except:
            print("error accessing and recording transactions")
            pass
        print("sleep {interval} s".format(interval = interval))
        time.sleep(interval)

def payment_in_transactions_CSV(
    reference    = None,
    value        = None,
    filepath_CSV = "RBS.csv"
    ):

    """
    Access transactions recorded in CSV and load them as a DataFrame. Search
    the "description" field of the DataFrame for a specified reference. If the
    reference is found, check the value of the payment is correct by comparing
    the specified value with the "amount" field.
    """

    if not reference or not value:
        return False
    if not os.path.isfile(filepath_CSV):
        print("file {filepath} not found".format(filepath = filepath_CSV))
        sys.exit()
    df = pd.read_csv(filepath_CSV)
    match = df[df["description"].str.contains(reference)]
    if not match.empty:
        return value == match["amount"].values[0]
    else:
        return False

def printout(
    filepath_credentials = None,
    log_out              = True,
    close_driver         = True,
    sleep_time           = 20
    ):

    status = account_status(
        filepath_credentials = filepath_credentials,
        log_out              = log_out,
        close_driver         = close_driver,
        sleep_time           = sleep_time
    )
    if status:
        text = "\n\nbalance: " + str(status["balance"]) + "\n" +\
               str(status["transactions"])
    else:
        text = "error"
    return text

def find_ordinal(text):

    match = re.search(r"(^|\W)[0-9]*([04-9]?((1st|2nd|3rd|[04-9]th))|(1[1-3]th))", text)
    return None if match is None else int(match.group(0)[:-2])

def currency_string_to_float(amount):

    """
    Convert a string of the form "£2,718.28" or "-£27.18" to a float with
    appropriate sign.
    """

    try:
        value = float(decimal.Decimal(re.sub(r"[^\d.]", "", amount)))
        if amount[0] == "-":
            value = -value
        return value
    except:
        pass
    return float(0)



# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# denarius                                                                     #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program provides currency and other utilities.                          #
#                                                                              #
# copyright (C) 2017 William Breaden Madden                                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################
"""

from __future__ import division
import ast
import datetime
import functools
import io
import json
import os
import pprint
import requests
import textwrap
import time
try:
    from urllib.request import urlopen
except:
    from urllib2 import urlopen
import sys

import currency_converter
import dataset
import datavision
import denarius.arbitrage
import matplotlib.pyplot as plt
import numpy
import pandas as pd
import pyprel
import shijian

name    = "denarius"
version = "2018-04-16T0034Z"

def prices_Bitcoin_Ethereum(
    convert_to_floats = True
    ):
    """
    Return a dictionary of prices for BTC and ETH as measured in EUR, USD and
    BTC using Bitstamp as a data source.
    """
    prices = {}
    prices["BTC_EUR"] = requests.get("https://www.bitstamp.net/api/v2/ticker/btceur").json()
    prices["BTC_USD"] = requests.get("https://www.bitstamp.net/api/v2/ticker/btcusd").json()
    prices["ETH_EUR"] = requests.get("https://www.bitstamp.net/api/v2/ticker/etheur").json()
    prices["ETH_USD"] = requests.get("https://www.bitstamp.net/api/v2/ticker/ethusd").json()
    prices["ETH_BTC"] = requests.get("https://www.bitstamp.net/api/v2/ticker/ethbtc").json()
    if convert_to_floats:
        for pair in prices:
            for price in prices[pair]:
                prices[pair][price] = float(prices[pair][price])
    return prices

def last_prices_Bitcoin_Ethereum():
    """
    Return a string presenting the last prices of BTC and ETH as measured in
    ETH, USD and BTC using Bitstamp as a data source. The string is intended for
    printing to terminal.
    """
    prices = prices_Bitcoin_Ethereum()
    text = textwrap.dedent(
    """
    BTC_EUR: {BTC_EUR_last}
    BTC_USD: {BTC_USD_last}
    
    ETH_EUR: {ETH_EUR_last}
    ETH_USD: {ETH_USD_last}
    
    ETH_BTC: {ETH_BTC_last}
    """.format(
        BTC_EUR_last = prices["BTC_EUR"]["last"],
        BTC_USD_last = prices["BTC_USD"]["last"],
        ETH_EUR_last = prices["ETH_EUR"]["last"],
        ETH_USD_last = prices["ETH_USD"]["last"],
        ETH_BTC_last = prices["ETH_BTC"]["last"]
    ))
    return text

def value_Bitcoin(
    price    = "last",
    currency = "EUR"
    ):
    return ticker_Bitcoin(
        currency = currency
    )[price]
    if day:
        URL = URL_day
    if hour:
        URL = URL_hour
    # Access a Bitcoin ticker.
    file_URL                = urlopen(URL)
    data_string             = file_URL.read().decode("utf-8")
    # Convert the data from string to dictionary.
    data_dictionary_strings = ast.literal_eval(data_string)
    #
    #    example hour ticker:
    #    {
    #        'ask':       999.31,
    #        'bid':       998.01,
    #        'high':      1000.6,
    #        'last':      999.31,
    #        'low':       996.67,
    #        'open':      990.0,
    #        'timestamp': 1487600377.0,
    #        'volume':    21.11748617,
    #        'vwap':      998.34
    #    }
    #
    #    example day ticker:
    #    {
    #        'ask':       999.31,
    #        'bid':       998.01,
    #        'high':      1004.0,
    #        'last':      999.31,
    #        'low':       985.31,
    #        'open':      990.0,
    #        'timestamp': 1487600340.0,
    #        'volume':    324.21451971,
    #        'vwap':      996.27
    #    }
    #
    # Convert numbers from strings to floats.
    data_dictionary = dict()
    for key in data_dictionary_strings:
        data_dictionary[key] = float(data_dictionary_strings[key])
    if currency != "EUR":
        # Convert currency EUR to requested currency.
        data_dictionary_currency = dict()
        converter_currency = currency_converter.CurrencyConverter()
        for key in data_dictionary:
            if key == "timestamp":
                data_dictionary_currency[key] = data_dictionary[key]
            else:
                data_dictionary_currency[key] =\
                    converter_currency.convert(
                        data_dictionary_strings[key],
                        "EUR",
                        currency
                    )
        return data_dictionary_currency
    return data_dictionary

def fluctuation_value_Bitcoin(
    days                      = 5,
    factor_standard_deviation = 1.6,
    details                   = False
    ):
    data_Bitcoin = data_historical_Bitcoin(
        days        = days,
        return_list = True
    )
    value_current_Bitcoin = value_Bitcoin()
    values_Bitcoin = [float(element[1]) for element in data_Bitcoin]
    mean_Bitcoin               = numpy.array(values_Bitcoin).mean()
    standard_deviation_Bitcoin = numpy.array(values_Bitcoin).std()
    low_Bitcoin  =\
        mean_Bitcoin - factor_standard_deviation * standard_deviation_Bitcoin
    high_Bitcoin =\
        mean_Bitcoin + factor_standard_deviation * standard_deviation_Bitcoin
    if details:
        print("Bitcoin values from last {days} days:\n\n{values}\n".format(
            days   = days,
            values = ", ".join([str(element) for element in values_Bitcoin])
        ))
        print("high bound Bitcoin:    {value}".format(value = high_Bitcoin))
        print("Bitcoin current value: {value}".format(
            value = value_current_Bitcoin
        ))
        print("low bound Bitcoin:     {value}".format(value = low_Bitcoin))
    if low_Bitcoin <= value_current_Bitcoin <= high_Bitcoin:
        return False
    else:
        return True

def fluctuation_value_LocalBitcoins(
    days                            = 3,
    factor_standard_deviation       = 1.6,
    details                         = False,
    filename_database_LocalBitcoins = "database_LocalBitcoins.db",
    ):
    datetime_time_start = datetime.datetime.utcnow() - datetime.timedelta(days = days)
    # get last few days of LocalBitcoins values from database
    values_LocalBitcoins = []
    database_LocalBitcoins = access_database(filename = filename_database_LocalBitcoins)
    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            values_LocalBitcoins.append(ast.literal_eval(row["values_GBP"])[0])
    # reverse order so latest first
    values_LocalBitcoins = values_LocalBitcoins[::-1]
    # current value
    value_current_LocalBitcoins = values_Bitcoin_LocalBitcoin()[0]
    mean_Bitcoin               = numpy.array(values_LocalBitcoins).mean()
    standard_deviation_Bitcoin = numpy.array(values_LocalBitcoins).std()
    low_Bitcoin  =\
        mean_Bitcoin - factor_standard_deviation * standard_deviation_Bitcoin
    high_Bitcoin =\
        mean_Bitcoin + factor_standard_deviation * standard_deviation_Bitcoin
    if details:
        print("high bound LocalBitcoins:    {value}".format(value = high_Bitcoin))
        print("LocalBitcoins current value: {value}".format(
            value = value_current_LocalBitcoins
        ))
        print("low bound LocalBitcoins:     {value}".format(value = low_Bitcoin))
    if low_Bitcoin <= value_current_LocalBitcoins <= high_Bitcoin:
        return False
    else:
        return True

def value_prediction_linear_Bitcoin(
    days_past   = 5, # past days data from which prediction is made
    days_future = 2, # number of days from now at which to predict value
    currency    = "EUR"
    ):
    data_Bitcoin = data_historical_Bitcoin(
        currency    = currency,
        days        = days_past,
        return_list = True
    )
    values_Bitcoin = [float(element[1]) for element in data_Bitcoin]
    data = [[day, value] for day, value in zip(list(range(-days_past, 0)), values_Bitcoin)]
    model_values = shijian.model_linear(data = data)
    b0 = model_values[0]
    b1 = model_values[1]
    x = days_future
    y = b0 + b1 * x
    return y

def ticker_Bitcoin(
    URL_hour = "https://www.bitstamp.net/api/v2/ticker/btceur",
    URL_day  = "https://www.bitstamp.net/api/v2/ticker_hour/btceur",
    currency = "EUR",
    hour     = True,
    day      = False
    ):
    """
    Return a dictionary with Bitcoin ask, bid, vwap, open, last, high and low
    prices in a specified currency (by default, EUR), together with a timestamp
    and volume, for either the last hour (by default) or the last day.

    example hour ticker:
    
    {
        'ask':       999.31,
        'bid':       998.01,
        'high':      1000.6,
        'last':      999.31,
        'low':       996.67,
        'open':      990.0,
        'timestamp': 1487600377.0,
        'volume':    21.11748617,
        'vwap':      998.34
    }

    """
    if day:
        URL = URL_day
    if hour:
        URL = URL_hour
    # Access a Bitcoin ticker.
    file_URL                = urlopen(URL)
    data_string             = file_URL.read().decode("utf-8")
    # Convert the data from string to dictionary.
    data_dictionary_strings = ast.literal_eval(data_string)
    # Convert numbers from strings to floats.
    data_dictionary = dict()
    for key in data_dictionary_strings:
        data_dictionary[key] = float(data_dictionary_strings[key])
    if currency != "EUR":
        # Convert currency EUR to requested currency.
        data_dictionary_currency = dict()
        converter_currency = currency_converter.CurrencyConverter()
        for key in data_dictionary:
            if key == "timestamp":
                data_dictionary_currency[key] = data_dictionary[key]
            else:
                data_dictionary_currency[key] =\
                    converter_currency.convert(
                        data_dictionary_strings[key],
                        "EUR",
                        currency
                    )
        return data_dictionary_currency
    return data_dictionary

def data_historical_Bitcoin(
    URL               = "https://api.coindesk.com/v1/bpi/historical/close.json",
    currency          = "EUR",
    date_start        = None, # YYYY-MM-DD
    date_stop         = None, # YYYY-MM-DD
    days              = None, # last days (start/stop dates alternative)
    return_list       = False,
    return_UNIX_times = False,
    sort_reverse      = False
    ):
    if days:
        time_current = datetime.datetime.utcnow()
        date_stop    = time_current.strftime("%Y-%m-%d")
        date_start   = (time_current -\
                       datetime.timedelta(days = days)).strftime("%Y-%m-%d")
    # Construct the URL using the API (http://www.coindesk.com/api/).
    URL = URL + "?currency=" + currency
    if date_start is not None and date_stop is not None:
        URL = URL + "&start=" + date_start + "&end=" + date_stop
    # Access the online data.
    file_URL                = urlopen(URL)
    data_string             = file_URL.read().decode("utf-8")
    # Convert the data from string to dictionary.
    data_dictionary_strings = ast.literal_eval(data_string)
    if return_list or return_UNIX_times:
        data_dictionary_list = list()
        for key in data_dictionary_strings["bpi"]:
            if return_UNIX_times:
                date = int(
                    time.mktime(
                        datetime.datetime.strptime(
                            key,
                            "%Y-%m-%d"
                        ).timetuple()
                    )
                )
            else:
                date = key
            data_dictionary_list.append(
                [date, float(data_dictionary_strings["bpi"][key])]
            )
        # sort
        data_dictionary_list_tmp = sorted(
            data_dictionary_list,
            key = lambda data_dictionary_list: (
                      data_dictionary_list[0],
                      data_dictionary_list[1]
                  ),
            reverse = sort_reverse
        )
        data_dictionary_list = data_dictionary_list_tmp
        return data_dictionary_list
    else:
        return data_dictionary_strings

def table_Bitcoin(
    currency   = "EUR",
    date_start = None, # YYYY-MM-DD
    date_stop  = None, # YYYY-MM-DD
    UNIX_times = False
    ):
    # Get Bitcoin value data.
    data = data_historical_Bitcoin(
        currency          = currency,
        date_start        = None, # YYYY-MM-DD
        date_stop         = None, # YYYY-MM-DD
        return_UNIX_times = UNIX_times,
        return_list       = True
    )
    # Return a table of the Bitcoin value data.
    table_contents = [[
                         "time",
                         "Bitcoin value ({currency})".format(
                             currency = currency
                         )
                     ]]
    table_contents.extend(data)
    table = pyprel.Table(
                contents = table_contents
            )
    return table

def save_graph_Bitcoin(
    currency   = "EUR",
    filename   = None,
    directory  = ".",
    overwrite  = True,
    date_start = None, # YYYY-MM-DD
    date_stop  = None, # YYYY-MM-DD
    days       = None  # last days (start/stop dates alternative)
    ):
    """
    Save a matplotlib graph of Bitcoin values.
    """
    if filename is None:
        filename = "Bitcoin_value_{currency}_versus_time.png".format(
            currency = currency
        )
    data = data_historical_Bitcoin(
        currency          = currency,
        date_start        = date_start, # YYYY-MM-DD
        date_stop         = date_stop,  # YYYY-MM-DD
        days              = days,
        return_UNIX_times = True
    )
    datavision.save_graph_matplotlib(
        values       = data,
        title_axis_x = "time",
        title_axis_y = "value ({currency})".format(currency = currency),
        filename     = filename,
        directory    = directory,
        overwrite    = overwrite,
        line         = True,
        line_width   = 0.5,
        time_axis_x  = True
    )

def save_graph_LocalBitcoins(
    filename          = None,
    directory         = ".",
    overwrite         = True,
    filename_database = "database_LocalBitcoins.db"
    ):
    """
    Save a matplotlib graph of LocalBitcoins lowest GBP values.
    """
    if filename is None:
        filename = "LocalBitcoins_Bitcoin_lowest_price_GBP_versus_time.png"
    database = access_database(filename = filename_database)
    data = []
    for row in database["LocalBitcoins"]:
        data.append([row["time_UNIX"], ast.literal_eval(row["values_GBP"])[0]])
    datavision.save_graph_matplotlib(
        values       = data,
        title_axis_x = "time",
        title_axis_y = "LBC low (GBP)",
        filename     = filename,
        directory    = directory,
        overwrite    = overwrite,
        line         = True,
        line_width   = 0.5,
        time_axis_x  = True,
        time_style   = "%Y-%m-%dT%H%MZ", # e.g. "%Y-%m-%d", "%Y-%m-%dT%H%MZ",
        font_size    = 8
    )

def save_graph_Bitcoin_LocalBitcoins(
    filename                        = "Bitcoin_LocalBitcoins_lowest_price_GBP.png",
    directory                       = ".",
    overwrite                       = True,
    filename_database_Bitcoin       = "database_Bitcoin_GBP.db",
    filename_database_LocalBitcoins = "database_LocalBitcoins.db",
    time_start                      = "2017-03-08T1436Z" # YYYY-MM-DDTHHMMZ
    ):

    """
    Save a matplotlib graph of Bitcoin GBP values versus LocalBitcoins lowest
    GBP values.
    """

    x_Bitcoin       = []
    y_Bitcoin       = []
    x_LocalBitcoins = []
    y_LocalBitcoins = []

    database_Bitcoin       = access_database(filename = filename_database_Bitcoin)
    database_LocalBitcoins = access_database(filename = filename_database_LocalBitcoins)

    datetime_time_start = datetime.datetime.strptime(time_start, "%Y-%m-%dT%H%MZ")

    for row in database_Bitcoin["Bitcoin"]:
        row_time = datetime.datetime.fromtimestamp(row["time"]) + datetime.timedelta(hours = 24)
        if row_time >= datetime_time_start:
            x_Bitcoin.append(row_time)
            y_Bitcoin.append(row["value"])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins.append(row_time)
            y_LocalBitcoins.append(ast.literal_eval(row["values_GBP"])[0])

    datavision.save_multigraph_2D_matplotlib(
        variables_x      = [x_Bitcoin, x_LocalBitcoins],
        variables_y      = [y_Bitcoin, y_LocalBitcoins],
        variables_names  = ["Bitcoin", "LocalBitcoins"],
        title            = "Bitcoin values versus LocalBitcoins lowest price",
        title_axis_x     = "time",
        title_axis_y     = "Bitcoin value (GBP)",
        filename         = filename,
        directory        = directory,
        overwrite        = overwrite,
        LaTeX            = False,
        markers          = False,
        marker_size      = 0.8,
        line             = True,
        line_width       = 0.5,
        palette_name     = "palette1",
        time_axis_x      = True,
        time_style       = "%Y-%m-%d",
        font_size        = 10
    )

def save_graphs_Bitcoin_LocalBitcoins(
    filename                        = "Bitcoin_LocalBitcoins_prices_GBP.png",
    directory                       = ".",
    overwrite                       = True,
    filename_database_Bitcoin       = "database_Bitcoin_GBP.db",
    filename_database_LocalBitcoins = "database_LocalBitcoins.db",
    time_start                      = "2017-03-08T1436Z" # YYYY-MM-DDTHHMMZ
    ):

    """
    Save a matplotlib graph of the LocalBitcoins 5 lowest GBP values and Bitcoin
    value.
    """

    x_Bitcoin         = []
    y_Bitcoin         = []

    x_LocalBitcoins_1 = []
    y_LocalBitcoins_1 = []

    x_LocalBitcoins_2 = []
    y_LocalBitcoins_2 = []

    x_LocalBitcoins_3 = []
    y_LocalBitcoins_3 = []

    x_LocalBitcoins_4 = []
    y_LocalBitcoins_4 = []

    x_LocalBitcoins_5 = []
    y_LocalBitcoins_5 = []

    database_Bitcoin       = access_database(filename = filename_database_Bitcoin)
    database_LocalBitcoins = access_database(filename = filename_database_LocalBitcoins)

    datetime_time_start = datetime.datetime.strptime(time_start, "%Y-%m-%dT%H%MZ")

    for row in database_Bitcoin["Bitcoin"]:
        row_time = datetime.datetime.fromtimestamp(row["time"]) + datetime.timedelta(hours = 24)
        if row_time >= datetime_time_start:
            x_Bitcoin.append(row_time)
            y_Bitcoin.append(row["value"])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_1.append(row_time)
            y_LocalBitcoins_1.append(ast.literal_eval(row["values_GBP"])[0])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_2.append(row_time)
            y_LocalBitcoins_2.append(ast.literal_eval(row["values_GBP"])[1])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_3.append(row_time)
            y_LocalBitcoins_3.append(ast.literal_eval(row["values_GBP"])[2])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_4.append(row_time)
            y_LocalBitcoins_4.append(ast.literal_eval(row["values_GBP"])[3])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_5.append(row_time)
            y_LocalBitcoins_5.append(ast.literal_eval(row["values_GBP"])[4])

    datavision.save_multigraph_2D_matplotlib(
        variables_x      = [
                           x_Bitcoin,
                           x_LocalBitcoins_1,
                           x_LocalBitcoins_2,
                           x_LocalBitcoins_3,
                           x_LocalBitcoins_4,
                           x_LocalBitcoins_5
                           ],
        variables_y      = [
                           y_Bitcoin,
                           y_LocalBitcoins_1,
                           y_LocalBitcoins_2,
                           y_LocalBitcoins_3,
                           y_LocalBitcoins_4,
                           y_LocalBitcoins_5
                           ],
        variables_names  = [
                           "Bitcoin",
                           "LocalBitcoins 1",
                           "LocalBitcoins 2",
                           "LocalBitcoins 3",
                           "LocalBitcoins 4",
                           "LocalBitcoins 5"
                           ],
        title            = "Bitcoin value versus LocalBitcoins lowest prices",
        title_axis_x     = "time",
        title_axis_y     = "Bitcoin value (GBP)",
        filename         = filename,
        directory        = directory,
        overwrite        = overwrite,
        LaTeX            = False,
        markers          = False,
        marker_size      = 0.8,
        line             = True,
        line_width       = 0.3,
        palette_name     = "palette1",
        time_axis_x      = True,
        time_style       = "%Y-%m-%d",
        font_size        = 10
    )

def save_graphs_Bitcoin_LocalBitcoins_Bollinger_bands(
    window                          = 20,
    directory                       = ".",
    overwrite                       = True,
    filename_database_Bitcoin       = "database_Bitcoin_GBP.db",
    filename_database_LocalBitcoins = "database_LocalBitcoins.db",
    time_start                      = "2017-03-08T1436Z", # YYYY-MM-DDTHHMMZ
    line_width                      = 0.4
    ):

    """
    Save matplotlib graphs of the LocalBitcoins 5 lowest GBP values and Bitcoin
    value.
    """

    filenames = [
        "Bitcoin_Bollinger_bands.png",
        "LocalBitcoins_1_Bollinger_bands.png",
        "LocalBitcoins_2_Bollinger_bands.png",
        "LocalBitcoins_3_Bollinger_bands.png",
        "LocalBitcoins_4_Bollinger_bands.png",
        "LocalBitcoins_5_Bollinger_bands.png",
    ]

    x_Bitcoin         = []
    y_Bitcoin         = []

    x_LocalBitcoins_1 = []
    y_LocalBitcoins_1 = []

    x_LocalBitcoins_2 = []
    y_LocalBitcoins_2 = []

    x_LocalBitcoins_3 = []
    y_LocalBitcoins_3 = []

    x_LocalBitcoins_4 = []
    y_LocalBitcoins_4 = []

    x_LocalBitcoins_5 = []
    y_LocalBitcoins_5 = []

    database_Bitcoin       = access_database(filename = filename_database_Bitcoin)
    database_LocalBitcoins = access_database(filename = filename_database_LocalBitcoins)

    datetime_time_start = datetime.datetime.strptime(time_start, "%Y-%m-%dT%H%MZ")

    for row in database_Bitcoin["Bitcoin"]:
        row_time = datetime.datetime.fromtimestamp(row["time"]) + datetime.timedelta(hours = 24)
        if row_time >= datetime_time_start:
            x_Bitcoin.append(row_time)
            y_Bitcoin.append(row["value"])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_1.append(row_time)
            y_LocalBitcoins_1.append(ast.literal_eval(row["values_GBP"])[0])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_2.append(row_time)
            y_LocalBitcoins_2.append(ast.literal_eval(row["values_GBP"])[1])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_3.append(row_time)
            y_LocalBitcoins_3.append(ast.literal_eval(row["values_GBP"])[2])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_4.append(row_time)
            y_LocalBitcoins_4.append(ast.literal_eval(row["values_GBP"])[3])

    for row in database_LocalBitcoins["LocalBitcoins"]:
        row_time = datetime.datetime.fromtimestamp(row["time_UNIX"])
        if row_time >= datetime_time_start:
            x_LocalBitcoins_5.append(row_time)
            y_LocalBitcoins_5.append(ast.literal_eval(row["values_GBP"])[4])

    datavision.save_plot_Bollinger_bands(
        x           = numpy.array(x_Bitcoin),
        y           = numpy.array(y_Bitcoin),
        window      = window,
        title       = "Bitcoin",
        time_axis_x = True,
        line_width  = line_width,
        filename    = filenames[0],
        directory   = directory
    )

    datavision.save_plot_Bollinger_bands(
        x           = numpy.array(x_LocalBitcoins_1),
        y           = numpy.array(y_LocalBitcoins_1),
        window      = window,
        title       = "LocalBitcoins 1",
        time_axis_x = True,
        line_width  = line_width,
        filename    = filenames[1],
        directory   = directory
    )

    datavision.save_plot_Bollinger_bands(
        x           = numpy.array(x_LocalBitcoins_2),
        y           = numpy.array(y_LocalBitcoins_2),
        window      = window,
        title       = "LocalBitcoins 2",
        time_axis_x = True,
        line_width  = line_width,
        filename    = filenames[2],
        directory   = directory
    )

    datavision.save_plot_Bollinger_bands(
        x           = numpy.array(x_LocalBitcoins_3),
        y           = numpy.array(y_LocalBitcoins_3),
        window      = window,
        title       = "LocalBitcoins 3",
        time_axis_x = True,
        line_width  = line_width,
        filename    = filenames[3],
        directory   = directory
    )

    datavision.save_plot_Bollinger_bands(
        x           = numpy.array(x_LocalBitcoins_4),
        y           = numpy.array(y_LocalBitcoins_4),
        window      = window,
        title       = "LocalBitcoins 4",
        time_axis_x = True,
        line_width  = line_width,
        filename    = filenames[4],
        directory   = directory
    )

    datavision.save_plot_Bollinger_bands(
        x           = numpy.array(x_LocalBitcoins_5),
        y           = numpy.array(y_LocalBitcoins_5),
        window      = window,
        title       = "LocalBitcoins 5",
        time_axis_x = True,
        line_width  = line_width,
        filename    = filenames[5],
        directory   = directory
    )

def save_graph_LocalBitcoins_days(
    filename          = "LocalBitcoins_Bitcoin_lowest_price_GBP_days.png",
    directory         = ".",
    overwrite         = True,
    filename_database = "database_LocalBitcoins.db"
    ):
    """
    Save a matplotlib graph of daily LocalBitcoins 5 lowest GBP values.
    """
    if filename is None:
        filename = "LocalBitcoins_Bitcoin_lowest_price_GBP_days.png"
    database = access_database(filename = filename_database)
    data_datetime         = []
    data_time_through_day = []
    data_value            = []
    for row in database["LocalBitcoins"]:
        time_row = datetime.datetime.fromtimestamp(row["time_UNIX"])
        time_through_day =\
            time_row -\
            datetime.datetime.combine(time_row.date(), datetime.time())
        data_datetime.append(time_row)
        data_time_through_day.append((time_through_day.seconds / 86400) * 24)
        data_value.append(ast.literal_eval(row["values_GBP"])[0])
    data                         = pd.DataFrame()
    data["value"]                = data_value
    data["time_through_day"]     = data_time_through_day
    data.index                   = data_datetime
    data.index.name              = "datetime"
    data["value_day_normalized"] = data["value"].div(data.resample("D")["value"].transform("sum"))
    # trim
    data_plot = []
    for data_time_through_day, data_value in zip(data["time_through_day"], data["value_day_normalized"]):
        if data_value < 0.025:
            data_plot.append([data_time_through_day, data_value])
    #data_plot = [[data_time_through_day, data_value] for data_time_through_day, data_value in zip(data["time_through_day"], data["value_day_normalized"])]
    datavision.save_graph_matplotlib(
        values       = data_plot,
        title_axis_x = "time (hours)",
        title_axis_y = "LBC low (GBP, normalised to unity)",
        filename     = filename,
        directory    = directory,
        overwrite    = overwrite,
        line         = False,
        line_width   = 0.5,
        #time_axis_x  = True,
        #time_style   = "%H%MZ", # e.g. "%Y-%m-%d", "%Y-%m-%dT%H%MZ",
        font_size    = 8,
        marker_size  = 2
    )

def save_graph_LocalBitcoins_weeks(
    filename          = "LocalBitcoins_Bitcoin_lowest_price_GBP_weeks.png",
    directory         = ".",
    overwrite         = True,
    filename_database = "database_LocalBitcoins.db",
    normalize         = True
    ):
    """
    Save a matplotlib graph of weekly LocalBitcoins 5 lowest GBP values.
    """
    if filename is None:
        filename = "LocalBitcoins_Bitcoin_lowest_price_GBP_weeks.png"
    database = access_database(filename = filename_database)
    data = []
    for row in database["LocalBitcoins"]:
        data.append(
            [
                datetime.datetime.fromtimestamp(row["time_UNIX"]),
                ast.literal_eval(row["values_GBP"])[0]
            ]
        )
    df = pd.DataFrame(data, columns = ["time", "LBC1"])
    df["weekday"]          = df["time"].dt.weekday
    df["weekday_name"]     = df["time"].dt.weekday_name
    df["time_through_day"] = df["time"].map(lambda x: x - datetime.datetime.combine(x.date(), datetime.time()))

    def days_through_week(row):
        return row["weekday"] + row["time_through_day"] / (24 * numpy.timedelta64(1, "h"))

    df["days_through_week"] = df.apply(lambda row: days_through_week(row), axis = 1)
    #data_plot = df.as_matrix(columns = ["days_through_week", "LBC1"]).tolist()
    #datavision.save_graph_matplotlib(
    #    values       = data_plot,
    #    title_axis_x = "days through week",
    #    title_axis_y = "LBC low (GBP)", #"LBC low (GBP, normalised to unity)",
    #    filename     = filename,
    #    directory    = directory,
    #    overwrite    = overwrite,
    #    line         = False,
    #    line_width   = 0.5,
    #    #time_axis_x  = True,
    #    #time_style   = "%H%MZ", # e.g. "%Y-%m-%d", "%Y-%m-%dT%H%MZ",
    #    font_size    = 8,
    #    marker_size  = 2
    #)
    datasets = []
    dataset = []
    previous_days_through_week = 0
    for days_through_week, LBC1 in zip(df["days_through_week"], df["LBC1"]):
        if abs(days_through_week - previous_days_through_week) < 5:
            dataset.append([days_through_week, LBC1])
        else:
            datasets.append(dataset)
            dataset = []
        previous_days_through_week = days_through_week
    plt.ioff()
    fig = plt.figure(figsize = (15, 8))
    for dataset in datasets:
        x = [datum[0] for datum in dataset]
        y = [datum[1] for datum in dataset]
        if normalize:
            y = datavision.normalize_to_range(y)
        plt.plot(
            x,
            y,
            linestyle = "-",
            linewidth = 1.3
        )
    #plt.axes().set_aspect(1 / plt.axes().get_data_ratio())
    plt.ylabel("LBC low (GBP)")
    plt.xticks(
        [
            0.5,
            1.5,
            2.5,
            3.5,
            4.5,
            5.5,
            6.5
        ],
        [
            "Monday",
            "Tuesday",
            "Wednesday",
            "Thursday",
            "Friday",
            "Saturday",
            "Sunday"
        ]
    )
    if not os.path.exists(directory):
        os.makedirs(directory)
    plt.savefig(
        directory + "/" + filename,
        dpi = 700
    )
    plt.close()

def graph_TTY_Bitcoin(
    currency   = "EUR",
    date_start = None, # YYYY-MM-DD
    date_stop  = None, # YYYY-MM-DD
    days       = None  # last days (start/stop dates alternative)
    ):
    """
    Return a datavision TTY terminal graph of Bitcoin values.
    """
    data = data_historical_Bitcoin(
        currency          = currency,
        date_start        = date_start, # YYYY-MM-DD
        date_stop         = date_stop,  # YYYY-MM-DD
        days              = days,
        return_UNIX_times = True
    )
    x = [element[0] for element in data]
    y = [element[1] for element in data]
    plot = datavision.TTYFigure()
    tmp = plot.plot(
        x,
        y,
        marker = "_o",
        plot_slope = False
    )
    return tmp

def create_database(
    filename = None
    ):
    """
    Create an empty SQLite database.
    """
    os.system(
        "sqlite3 " + \
        filename + \
        " \"create table aTable(field1 int); drop table aTable;\""
    )

def access_database(
    filename = "database.db"
    ):
    """
    Return a dataset connection to an SQLite database.
    """
    database = dataset.connect("sqlite:///" + str(filename))
    return database

def save_database_Bitcoin(
    filename   = "database_Bitcoin.db",
    currency   = "EUR",
    date_start = "2010-07-17",
    date_stop  = None
    ):
    if date_stop is None:
        date_stop = datetime.datetime.utcnow().strftime("%Y-%m-%d")
    data = data_historical_Bitcoin(
        currency          = currency,
        date_start        = date_start,
        date_stop         = date_stop,
        return_list       = True,
        return_UNIX_times = True
    )
    database = access_database(filename = filename)
    table    = database["Bitcoin"]
    progress = shijian.Progress()
    progress.engage_quick_calculation_mode()
    number_of_entries = len(data)
    for index, element in enumerate(data):
        table.insert(dict(
            time      = element[0],
            value     = element[1]
        ))
        print(progress.add_datum(fraction = index / number_of_entries))

def table_database(
    filename           = "database.db",
    name_table         = "Bitcoin",
    include_attributes = None,
    rows_limit         = None
    ):
    """
    Return a pyprel table from an SQLite database.
    """
    database = access_database(filename = filename)
    return pyprel.Table(
        contents = pyprel.table_dataset_database_table(
            table              = database[name_table],
            include_attributes = include_attributes,
            rows_limit         = rows_limit
        )
    )

def quote_Kraken(
    asset_1 = "XBT",
    asset_2 = "EUR",
    URL     = "https://api.kraken.com/0/public/Ticker?pair="
    ):
    """
    Access the Kraken API and return the value of a specified Kraken asset (by
    default XBT (BTC)) in terms of a specified Kraken asset (by default EUR) as
    a float.
    """
    try:
        results = requests.get(URL + asset_1 + asset_2).json()
        value   = results["result"]["X" + asset_1 + "Z" + asset_2]["c"][0]
        return float(value)
    except:
        print("error -- could not access Kraken API")
        return None

def quote_arbitrage_data_Kraken_LocalBitcoins_UK():
    """
    Get the last Kraken price in EUR. Convert this value to GBP using the
    current exchange rate. Get the current 5 lowest prices on LocalBitcoins UK
    in GBP. Divide the third lowest LocalBitcoins UK price by the last Kraken
    price in GBP. Return all of this data in a dictionary.
    """
    try:
        Kraken_last_price_EUR = quote_Kraken()
    except:
        print("error -- could not access Kraken API")
        return None
    try:
        Kraken_last_price_GBP = denarius.arbitrage.convert_currency(
                                    amount          = Kraken_last_price_EUR,
                                    currency_input  = "EUR",
                                    currency_output = "GBP"
                                )
    except:
        print("error -- could not convert currency")
        return None
    try:
        LBC                   = values_Bitcoin_LocalBitcoin()
    except:
        print("error -- could not access LocalBitcoins API")
        return None
    results = {
        "datetime":               datetime.datetime.utcnow(),
        "Kraken_last_price_EUR":  Kraken_last_price_EUR,
        "Kraken_last_price_GBP":  Kraken_last_price_GBP,
        "LBC_1_GBP":              LBC[0],
        "LBC_2_GBP":              LBC[1],
        "LBC_3_GBP":              LBC[2],
        "LBC_4_GBP":              LBC[3],
        "LBC_5_GBP":              LBC[4],
        "arbitrage_Kraken_LBC_1": LBC[0] / Kraken_last_price_GBP,
        "arbitrage_Kraken_LBC_2": LBC[1] / Kraken_last_price_GBP,
        "arbitrage_Kraken_LBC_3": LBC[2] / Kraken_last_price_GBP,
        "arbitrage_Kraken_LBC_4": LBC[3] / Kraken_last_price_GBP,
        "arbitrage_Kraken_LBC_5": LBC[4] / Kraken_last_price_GBP
    }
    return results

def loop_save_arbitrage_data_Kraken_LocalBitcoins_UK(
    filename = "arbitrage_Kraken_LocalBitcoins_UK.csv",
    interval = 900,
    verbose  = True
    ):
    """
    Loop save arbitrage data for Kraken and LocalBitcoins UK to a CSV file.
    """
    pp = pprint.PrettyPrinter(indent = 4, width = 1)
    while True:
        results = quote_arbitrage_data_Kraken_LocalBitcoins_UK()
        if results:
            df = pd.DataFrame()
            df = df.append(results, ignore_index = True)
            df.index = df["datetime"]
            del df["datetime"]
            if not os.path.isfile(filename):
                header = True
            else:
                header = False
            with open(filename, "a") as file_arbitrage:
                df.to_csv(file_arbitrage, header = header)
            if verbose:
                print("")
                pp.pprint(results)
        print("\nnext recording in {interval} s".format(interval = interval))
        time.sleep(interval)

def values_Bitcoin_LocalBitcoin(
    URL = "https://localbitcoins.com/buy-bitcoins-online/"\
          "GB/united-kingdom/national-bank-transfer/.json"
    ):
    file_URL    = urlopen(URL)
    data_string = file_URL.read().decode("utf-8")
    data_JSON   = json.loads(data_string)
    advertisements = data_JSON["data"]["ad_list"]
    advertisement_prices = []
    for advertisement in advertisements:
        advertisement_prices.append(float(advertisement["data"]["temp_price"]))
    advertisement_prices.sort()
    return advertisement_prices

def save_current_values_LocalBitcoins_to_database(
    filename   = "database_LocalBitcoins.db"
    ):
    # Data saved to the database is a UTC datetime timestamp, a UTC UNIX,
    # timestamp, the LocalBitcoins API string returned (JSON) and a list of the
    # current prices in GBP.
    timestamp      = datetime.datetime.utcnow()
    timestamp_UNIX = (timestamp -\
                     datetime.datetime.utcfromtimestamp(0)).total_seconds()
    # buying prices
    URL = "https://localbitcoins.com/buy-bitcoins-online/"\
          "GB/united-kingdom/national-bank-transfer/.json"
    file_URL    = urlopen(URL)
    data_string = file_URL.read().decode("utf-8")
    data_JSON   = json.loads(data_string)
    advertisements = data_JSON["data"]["ad_list"]
    advertisement_prices = []
    for advertisement in advertisements:
        advertisement_prices.append(float(advertisement["data"]["temp_price"]))
    advertisement_prices.sort()
    # selling prices
    URL = "https://localbitcoins.com/sell-bitcoins-online/"\
        "GB/united-kingdom/national-bank-transfer/.json"
    file_URL_sell    = urlopen(URL)
    data_string_sell = file_URL_sell.read().decode("utf-8")
    data_JSON_sell   = json.loads(data_string_sell)
    advertisements_sell = data_JSON_sell["data"]["ad_list"]
    advertisement_prices_sell = []
    for advertisement_sell in advertisements_sell:
        advertisement_prices_sell.append(float(advertisement_sell["data"]["temp_price"]))
    advertisement_prices_sell.sort()
    # save to database
    database = access_database(filename = filename)
    table    = database["LocalBitcoins"]
    table.insert(dict(
        time             = timestamp,
        time_UNIX        = timestamp_UNIX,
        JSON_GB_NBT      = str(data_string),
        JSON_GB_NBT_sell = str(data_string_sell),
        values_GBP       = str(advertisement_prices),
        values_GBP_sell  = str(advertisement_prices_sell)
    ))

def loop_save_current_values_LocalBitcoins_to_database(
    filename    = "database_LocalBitcoins.db",
    time_period = 1800, # seconds (30 minutes)
    verbose     = True
    ):
    while True:
        if verbose:
            print(
                "{time} save LocalBitcoins current data to database "\
                "{filename} (next save in {seconds} s)".format(
                    time     = datetime.datetime.utcnow(),
                    filename = filename,
                    seconds  = time_period
                )
            )
        save_current_values_LocalBitcoins_to_database(
            filename = filename
        )
        time.sleep(time_period)

def table_database_LocalBitcoins(
    filename           = "database_LocalBitcoins.db",
    name_table         = "LocalBitcoins",
    include_attributes = ["time", "time_UNIX", "values_GBP"],
    rows_limit         = None
    ):
    return table_database(
        filename           = filename,
        name_table         = name_table,
        include_attributes = include_attributes,
        rows_limit         = rows_limit
    )

def save_database_LocalBitcoins_to_DataFrame_PKL(
    filename_database = "database_LocalBitcoins.db",
    filename_PKL      = "LBC.pkl"
    ):
    """
    Save a subset of information of the LocalBitcoins SQLite database to a
    pickled DataFrame, specifically the 5 lowest GBP prices, with some
    additional variables, including weekday, weekday name, time through day and
    fraction through day.
    """
    datebase = dataset.connect("sqlite:///" + filename_database)
    data = []
    for row in database["LocalBitcoins"]:
        data.append(
            [
                datetime.datetime.fromtimestamp(row["time_UNIX"]),
                ast.literal_eval(row["values_GBP"])[0],
                ast.literal_eval(row["values_GBP"])[1],
                ast.literal_eval(row["values_GBP"])[2],
                ast.literal_eval(row["values_GBP"])[3],
                ast.literal_eval(row["values_GBP"])[4]
            ]
        )
    LBC = pd.DataFrame(
        data,
        columns = [
            "datetime",
            "LBC1",
            "LBC2",
            "LBC3",
            "LBC4",
            "LBC5"
        ]
    )
    LBC["weekday"]              = LBC["datetime"].dt.weekday
    LBC["weekday_name"]         = LBC["datetime"].dt.weekday_name
    LBC["time_through_day"]     = LBC["datetime"].map(lambda x: x - datetime.datetime.combine(x.date(), datetime.time()))
    LBC["fraction_through_day"] = LBC["time_through_day"].map(lambda x: x / datetime.timedelta(hours = 24))
    LBC.index = LBC["datetime"]
    del LBC["datetime"]
    LBC.index = pd.to_datetime(LBC.index.values)
    LBC.to_pickle(filename_PKL)

def instrument_DataFrame(
    instrument = None,
    date_start = "2010-01-01"
    ):
    """
    Access the following data of an instrument:

    - Open
    - High
    - Low
    - Close
    - Volume

    Return a DataFrame with the index set to datetime (DatetimeIndex) and with
    the columns Open, High, Low, Close, Volume and Instrument. Google Finance is
    used as a data source.
    """
    print("access instrument {instrument}".format(instrument = instrument))
    URL = "http://www.google.com/finance/historical"\
          "?q=NASDAQ:{instrument}"\
          "&startdate={date_start}"\
          "&output=csv".format(
              instrument = instrument,
              date_start = date_start
          )
    CSV = urlopen(URL).read()[3:].decode("utf-8").split()
    df = pd.read_csv(
        io.StringIO("\n".join(CSV)),
        na_values = None
    )
    df.index = df["Date"]
    del df["Date"]
    df.index = pd.to_datetime(df.index.values)
    df["Instrument"] = instrument
    return df

def merge_instrument_DataFrames(
    dfs          = None,
    change_names = True
    ):
    """
    Return a DataFrame of instrument DataFrames merged on the datetime index
    (DatetimeIndex). By default, the names of variables are changed for merging
    and the Instrument variable is removed because it is added to the changed
    names of the other variables.
    """
    if not change_names:
        return pd.concat(dfs)
    else:
        _dfs = []
        for df in dfs:
            instrument = df["Instrument"].values[0]
            df = df.rename(columns = {
                                         "Open":   "Open_"   + instrument,
                                         "High":   "High_"   + instrument,
                                         "Low":    "Low_"    + instrument,
                                         "Close":  "Close_"  + instrument,
                                         "Volume": "Volume_" + instrument
                                     }
                          )
            del df["Instrument"]
            _dfs.append(df)
        return functools.reduce(lambda left, right: pd.merge(left, right, left_index = True, right_index = True), _dfs)

def get_currencies(
    URL = "https://finance.yahoo.com/webservice/v1/symbols/allcurrencies/quote?format=json"
    ):
    """
    Return a DataFrame of currency prices expressed in other currencies using
    the Yahoo Finance API as a data source.
    """
    resources = requests.get(URL).json()["list"]["resources"]
    df = pd.DataFrame()
    for resource in resources:
        fields = resource["resource"]["fields"]
        df = df.append(
            {
                "datetime": datetime.datetime.strptime(fields["utctime"], "%Y-%m-%dT%H:%M:%S+0000"),
                "name":     fields["name"],
                "symbol":   fields["symbol"],
                "price":    fields["price"]
            },
            ignore_index = True
        )
    df.index = df["datetime"]
    del df["datetime"]
    return df

def convert_currency(
    value      = 1,
    currency_1 = "EUR",
    currency_2 = "GBP"
    ):
    #df = get_currencies()
    #rate_1 = float(df.query("(name == 'USD/" + currency_1 + "')")["price"].values[0])
    #rate_2 = float(df.query("(name == 'USD/" + currency_2 + "')")["price"].values[0])
    #result = float(value) * rate_2 / rate_1
    #return result
    return denarius.arbitrage.convert_currency(
        amount          = value,
        currency_input  = currency_1,
        currency_output = currency_2
    )

def printout_arbitrage_Kraken_LocalBitcoins_UK(
    filepath_CSV = "arbitrage_Kraken_LocalBitcoins_UK.csv",
    volume_EUR   = 9000
    ):
    if not os.path.isfile(filepath_CSV):
        print("no CSV file {filepath_CSV} found".format(filepath_CSV = filepath_CSV))
        return False
    else:
        try:
            _df = pd.read_csv(filepath_CSV)

            v = volume_EUR
            g = 100  # EUR minimum gain

            # costs
            #
            # Kraken EUR to BTC fee: 1.6 %
            # Kraken BTC withdraw:   0.0005 BTC (~4.53 EUR 2018-03-02T0242Z)
            # LBC sell fee:          1 %
            # TW GBP to EUR fee:     0.35 % + 0.8 GBP (~0.91 EUR)

            # loss factor is number by which volume amount can be multipled to get voluming resulting after fees etc.
            now = datetime.datetime.utcnow()
            if now.year == 2018 and now.month == 1: # no fees as a response to Kraken downtime
                loss_factor = (v - (0 * v) - (11.06) - (0.01 * v) - (0.0035 * v + 0.91)) / v # Kraken free trading for 2018-01
            else:
                loss_factor = (v - (0.016 * v) - (4.53) - (0.01 * v) - (0.0035 * v + 0.91)) / v
            
            required_arbitrage_factor = 2 - loss_factor + g / v

            current_arbitrage_factor = _df["arbitrage_Kraken_LBC_3"].values[-1]
            profit_with_current_arbitrage_factor = current_arbitrage_factor * (v * loss_factor) - v

            prices = prices_Bitcoin_Ethereum()

            text = textwrap.dedent(
            """
            
            {datetime}
            
            arbitrage_Kraken_LBC_3:   {arbitrage_Kraken_LBC_3} <---
            arbitrage_Kraken_LBC_2:   {arbitrage_Kraken_LBC_2}
            arbitrage_Kraken_LBC_1:   {arbitrage_Kraken_LBC_1}
            
            LBC_1:                    {LBC_1_GBP}
            LBC_2:                    {LBC_2_GBP}
            LBC_3:                    {LBC_3_GBP} <---
            LBC_4:                    {LBC_4_GBP}
            LBC_5:                    {LBC_5_GBP}
            
            Kraken_last_price_GBP:    {Kraken_last_price_GBP}
            Kraken_last_price_EUR:    {Kraken_last_price_EUR}
            
            BTC_EUR:                  {BTC_EUR_last}
            BTC_USD:                  {BTC_USD_last}
            ETH_EUR:                  {ETH_EUR_last}
            ETH_USD:                  {ETH_USD_last}
            ETH_BTC:                  {ETH_BTC_last}
            
            volume:                   {volume} EUR
            
            required arbitrage
            factor:                   {required_arbitrage_factor}
            
            profit with current
            arbitrage factor:         {profit_with_current_arbitrage_factor}
            
            trade?                    {conditional} <---
            """.format(
                datetime                             = _df["datetime"].values[-1],
                arbitrage_Kraken_LBC_3               = _df["arbitrage_Kraken_LBC_3"].values[-1],
                arbitrage_Kraken_LBC_2               = _df["arbitrage_Kraken_LBC_2"].values[-1],
                arbitrage_Kraken_LBC_1               = _df["arbitrage_Kraken_LBC_1"].values[-1],
                gain                                 = g,
                volume                               = v,
                required_arbitrage_factor            = required_arbitrage_factor,
                profit_with_current_arbitrage_factor = profit_with_current_arbitrage_factor,
                LBC_1_GBP                            = _df["LBC_1_GBP"].values[-1],
                LBC_2_GBP                            = _df["LBC_2_GBP"].values[-1],
                LBC_3_GBP                            = _df["LBC_3_GBP"].values[-1],
                LBC_4_GBP                            = _df["LBC_4_GBP"].values[-1],
                LBC_5_GBP                            = _df["LBC_5_GBP"].values[-1],
                Kraken_last_price_GBP                = _df["Kraken_last_price_GBP"].values[-1],
                Kraken_last_price_EUR                = _df["Kraken_last_price_EUR"].values[-1],
                conditional                          = "YES" if _df["arbitrage_Kraken_LBC_3"].values[-1] >= required_arbitrage_factor else "NO",
                BTC_EUR_last                         = prices["BTC_EUR"]["last"],
                BTC_USD_last                         = prices["BTC_USD"]["last"],
                ETH_EUR_last                         = prices["ETH_EUR"]["last"],
                ETH_USD_last                         = prices["ETH_USD"]["last"],
                ETH_BTC_last                         = prices["ETH_BTC"]["last"]
            ))
            return(text)
        except:
            pass
        return "error"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# denarius_graph_Bitcoin                                                       #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program displays a graph of Bitcoin values.                             #
#                                                                              #
# copyright (C) 2017 Will Breaden Madden, wbm@protonmail.ch                    #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help       display help message
    --version        display version and exit
    --currency=TEXT  currency                  [default: EUR]
    --days=INT       number of days to graph   [default: 60]
    --width=INT      interface width           [default: 900]
    --height=INT     interface height          [default: 900]
"""

name    = "denarius_graph_Bitcoin"
version = "2017-03-08T1411Z"
logo    = None

import docopt
import sys

import denarius
import PyQt5.QtCore
from PyQt5.QtWidgets import (
    QApplication,
    QLabel,
    QPushButton,
    QToolTip,
    QVBoxLayout,
    QWidget
)
from PyQt5 import QtGui
import shijian

def main(options):

    currency  = options["--currency"]
    days      = int(options["--days"])
    width     = int(options["--width"])
    height    = int(options["--height"])
    filename  = shijian.propose_filename() + ".png"
    directory = "/tmp/"

    denarius.save_graph_Bitcoin(
        currency  = currency,
        days      = days,
        filename  = filename,
        directory = directory
    )

    application = QApplication(sys.argv)
    window = Window(
        width     = width,
        height    = height,
        filename  = filename,
        directory = directory
    )
    sys.exit(application.exec_())

class Window(QWidget):

    def __init__(
        self,
        filename  = None,
        directory = ".",
        width     = 500,
        height    = 500
        ):

        super(Window, self).__init__()
        self.resize(height, width)
        self.layout = QVBoxLayout(self)
        self.add_image(
            filename  = filename,
            directory = directory
        )
        self.setFixedSize(self.width(), self.height())
        self.setStyleSheet(
        """
        QWidget{
            background: #ffffff;
        }
        """
        )
        self.show()

    def add_image(
        self,
        filename  = None,
        directory = "."
        ):

        image = QtGui.QPixmap(directory + filename)
        image = image.scaled(
                    self.width(),
                    self.height(),
                    PyQt5.QtCore.Qt.KeepAspectRatio
                )
        image_viewer = QLabel()
        image_viewer.setAlignment(PyQt5.QtCore.Qt.AlignCenter)
        image_viewer.setPixmap(image)
        self.layout.addWidget(image_viewer)

    def keyPressEvent(
        self,
        event
        ):

        if event.key() == PyQt5.QtCore.Qt.Key_Escape:
            self.close()

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
################################################################################
#                                                                              #
# denarius_loop_save_SlushPool                                                 #
#                                                                              #
################################################################################
#                                                                              #
# LICENCE INFORMATION                                                          #
#                                                                              #
# This program loop records SlushPool address data.                            #
#                                                                              #
# copyright (C) 2017 De2thDr, De2thDr@yandex.com, Will Breaden Madden          #
#                                                                              #
# This software is released under the terms of the GNU General Public License  #
# version 3 (GPLv3).                                                           #
#                                                                              #
# This program is free software: you can redistribute it and/or modify it      #
# under the terms of the GNU General Public License as published by the Free   #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# This program is distributed in the hope that it will be useful, but WITHOUT  #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or        #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for     #
# more details.                                                                #
#                                                                              #
# For a copy of the GNU General Public License, see                            #
# <http://www.gnu.org/licenses/>.                                              #
#                                                                              #
################################################################################
#                                                                              #
# CHANGELOG                                                                    #
#                                                                              #
# v0.1: Initial release                                                        #
# v0.2: Added html5lib explicit dependency.                                    #
#       Modified basicPlotMacro.C: Three instances of "char*" changed to       #
#       "const char*" as required by ISO C++11.                                #
# v0.3: Fixed exception occurring on payout - and now prints exception to      #
#       standard output too.                                                   #
#                                                                              #
################################################################################

usage:
    program [options]

options:
    -h, --help                 display help message
    --version                  display version and exit

    --addresses=TEXT           address or comma-separated addresses
    --interval=FLOAT           loop time interval (s)                            [default: 120]
    --alarm=FLOAT              alarm will activate if hashpower drops below      [default: 5900000]
    --silent                   turn off sounds
    --no_live_view             turn off live ROOT plot

    --slushtoken=TEXT          token you get from Slush's website under API
    --slushtokenfile=FILEPATH  Slush API token filepath                          [default: ~/.slushpool]
    --slushloginname=TEXT      your login name for Slush
    --slushworkername=TEXT     your worker's name at Slush

    --graphtimespan=FLOAT      the time interval (seconds) to use for the graph  [default: 86400]
    --file_CSV=FILENAME        filename for saving CSV data                      [default: SlushPool.csv]
"""

import docopt
from collections import OrderedDict
import json
import os
import requests
import sys
import time

from bs4 import BeautifulSoup
import html5lib

name    = "denarius_loop_save_SlushPool"
version = "2017-10-23T1107Z"
logo    = name

def main(options):

    addresses        =       options["--addresses"]
    interval         = float(options["--interval"])
    alarm            = float(options["--alarm"])
    silent           =       options["--silent"]
    no_live_view     =       options["--no_live_view"]
    slushtoken       =       options["--slushtoken"]
    filepath_token   =       options["--slushtokenfile"]
    slushloginname   =       options["--slushloginname"]
    slushworkername  =       options["--slushworkername"]
    graphtimespan    = float(options["--graphtimespan"])
    filename_CSV     =       options["--file_CSV"]

    filepath_token   = os.path.expanduser(os.path.expandvars(filepath_token))

    if not addresses:
        print("error -- no address specified")
        sys.exit()

    if not slushtoken:
        if os.path.isfile(filepath_token):
            file_token_contents =\
                [line.rstrip("\n") for line in open(filepath_token)]
            token = [line for line in file_token_contents if line]
            if token:
                slushtoken = token[0]
        else:
            print("error -- no Slush API token specified or found in token file. Get this from Slush's website.")
            sys.exit()

    if not slushloginname:
        print("error -- no Slush login name specified. Register at Slush's website.")
        sys.exit()

    if not slushworkername:
        print("error -- no Slush worker name specified. Register at Slush's website.")
        sys.exit()

    addresses = addresses.split(",")

    # Ensure dependencies.
    if not no_live_view:
        for program in ["root", "gv"]:
            if not which(program):
                print("error -- {program} not available".format(program = program))
                sys.exit()
    programs_sound = ["aplay", "afplay"]
    if not silent and not any([which(program) for program in programs_sound]):
        print("error -- no sound program found: " + ", ".join(programs_sound))

    #####################################################
    # Specify the variables we're interested in from Slush..
    #####################################################
    variables = OrderedDict([
        ("hashrate", "hash_rate_TH/s"),
        ("shares",     "shares")
    ])

    variablesOuter = OrderedDict([
        ("unconfirmed_reward",  "unconf_reward"),
        ("confirmed_reward",      "conf_reward")
    ])

    #####################################################
    # Define some variables to save/store values as we progress
    #####################################################
    sharesCheck=0
    totalReward=0.0
    totalRewardSave=0.0
    my_unconf_reward=0.0
    my_unconf_reward_save=0.0
    my_conf_reward=0.0
    my_conf_reward_save=0.0
    # NB: this stops the stupid InsecurePlatformWarning from SSL...
    requests.packages.urllib3.disable_warnings()
    paymentAmount=0.0
    blockfound=0
    firstLoop=1

    #####################################################
    # Begin to loop - do query and process result
    #####################################################
    while True:
        for address in addresses:
            try:
                URL         = "https://slushpool.com/accounts/profile/json/" + str(slushtoken)
                data_string = requests.get(URL, verify = False).text
                soup        = BeautifulSoup(data_string, "html5lib")
                data_JSON2  = json.loads(soup.findAll("body")[0].get_text())
                data_JSON   = data_JSON2["workers"][str(slushloginname) + "." + str(slushworkername)]

                #####################################################
                # First, and only once per GET request
                # Get a date/time in human format and also UNIX timestamp
                #####################################################
                humandate = os.system("date")
                timestamp = int(time.time())
                print("Timestamp: " + str(timestamp))

                #####################################################
                # Print the address and make it the first item in 'line'
                #####################################################
                print("\naddress: {address}".format(address = address))
                line = [address]

                #####################################################
                # Before we get the unconf_reward, save what we have
                # (could be 0) in case we found a block.
                # And set blockFound variables to 0.
                #####################################################
                my_unconf_reward_save = my_unconf_reward
                blockFound=0

                #####################################################
                # First loop over "variablesOuter" to get
                # unconf/conf rewards and add them
                #####################################################
                for variableOuter_key in list(variablesOuter.keys()):
                    # Get the value and then Print the list items found -- only those found which are also in the list "OrderedDict"!
                    valueOuter = data_JSON2[variableOuter_key]
                    if variableOuter_key == "unconfirmed_reward":
                        print("unconf_reward: " + str(valueOuter))
                        my_unconf_reward=valueOuter
                    if variableOuter_key == "confirmed_reward":
                        print("conf_reward:   " + str(valueOuter))
                        my_conf_reward=valueOuter

                totalReward=float(my_unconf_reward)+float(my_conf_reward)
                print("TOTAL REWARD:  " + str(totalReward))

                #####################################################
                # my_conf_reward can drop if there's a payout..
                #####################################################
                paymentAmount=0.0
                if float(my_conf_reward_save) > float(my_conf_reward):
                    print("PAYMENT HAS OCCURRED!")
                    paymentAmount = float(my_conf_reward_save) - float(my_conf_reward)
                    print("===========================================")
                    print("PAYMENT=" + str(paymentAmount))
                    print("===========================================")
                    print("my_conf_reward_save: " + str(my_conf_reward_save))
                    print("my_conf_reward: " + str(my_conf_reward_save))

                #####################################################
                #Save the totalReward here for use in the next iteration
                #####################################################
                my_conf_reward_save = float(my_conf_reward)

                #####################################################
                # Loop over the rest of the variables from the JSON
                #####################################################
                for variable_key in list(variables.keys()):

                    #####################################################
                    #Get the value and then Print the list items found -- only those found which are also in the list "OrderedDict"!
                    #####################################################
                    value = data_JSON[variable_key]
                    line.append(str(value))

                    #####################################################
                    # This traps a situation where there is a sharp/rapid
                    # and profound drop in hashrate...
                    #####################################################
                    if variable_key == "hashrate":
                        print("hashrate:      " + str(value))
                        if float(value) < alarm:
                            #####################################################
                            # User can add any type of alarm they like here.
                            #####################################################
                            if not silent:
                                play_sound(filename = "CarterAirRaidSiren.wav")
                            print("value < threshold CALL ALARM!")

                    if variable_key == "shares":
                        if int(value) >= sharesCheck:
                            sharesCheck=int(value)
                            print("Shares increased/stayed same... no block..."+str(sharesCheck))
                        else:
                            #####################################################
                            # User can add any type of celebration they like here.
                            #####################################################
                            if not silent:
                                play_sound(filename = "ateamhires.mp3")
                            print("SHARES HAVE REDUCED! WE HAVE A BLOCK :) ")
                            sharesCheck=int(value)
                            blockFound=1

                #####################################################
                # Store the timestamp and any rewards.
                #####################################################
                line.append(str(timestamp))
                line.append(str(my_unconf_reward))
                line.append(str(my_conf_reward))
                line.append(str(totalReward))
                line.append(str(paymentAmount))
                line.append(str(blockFound))

                #####################################################
                # Write to the .csv file.
                #####################################################
                with open(filename_CSV, "ab") as file_CSV:
                    file_CSV.write(",".join(line) + "\n")

                if not no_live_view:

                    #####################################################
                    # Run root macro in 'batch' mode (to stop focus stealing),
                    # and use -q to get the macro to exit after it runs.
                    #####################################################
                    os.system('root -l -q -b "basicPlotMacro.C(' + str(graphtimespan) + ', \\"SlushPool.csv\\")" &')

                    #####################################################
                    # Try to open the "eps" file using GhostView (gv)
                    # NB: The first time this code runs, there will be no .eps
                    # file until the root command "basicPlotMacro.C" has run.
                    # so I introduce a little 'sleep' command here to hope it
                    # will finish executing before we try to draw the result with 'gv'
                    ################a#####################################
                    if firstLoop == 1:
                        try:
                            print("Wait 5 seconds to allow root process to finish..")
                            time.sleep(5)
                            os.system('gv -watch SlushPool.eps&')
                            firstLoop=0
                        except:
                            print("Couldn't find / display .eps file..")

            #####################################################
            # Trap exceptions here
            #####################################################
            except Exception as e:
                pass
                print(e)
                #####################################################
                # User can add any type of alarm they like here.
                #####################################################
                if not silent:
                    play_sound(filename = "CarterAirRaidSiren.wav")
                print("AN EXCEPTION OCCURRED!")

        #####################################################
        # Sleep for a few seconds (interval)..
        #####################################################
        print("\nwait {interval} s".format(interval = interval))
        time.sleep(interval)

def play_sound(
    filename = None
    ):

    if which("aplay"):

        command = """
        aplay --quiet "{filename}" &
        """.format(
            filename = filename
        )

    elif which("afplay"):

        command = """
        afplay "{filename}" &
        """.format(
            filename = filename
        )

    os.system(command)

def which(
    program
    ):

    def is_exe(fpath):

        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)

    if fpath:

        if is_exe(program):

            return(program)

    else:

        for path in os.environ["PATH"].split(os.pathsep):

            path = path.strip('"')
            exe_file = os.path.join(path, program)

            if is_exe(exe_file):

                return exe_file

    return None

if __name__ == "__main__":
    options = docopt.docopt(__doc__)
    if options["--version"]:
        print(version)
        exit()
    main(options)

